summaryrefslogtreecommitdiff
path: root/dmenu.c
diff options
context:
space:
mode:
authorjustsaumit <justsaumit@draconyan.xyz>2022-03-02 09:11:58 +0530
committerjustsaumit <justsaumit@draconyan.xyz>2022-03-02 09:11:58 +0530
commit96a2605674e2621957af40b7b8eb450ccf2841e6 (patch)
treee340ad817ef9eb97705e06f199fbb526f064bbed /dmenu.c
parent6ddf8b2836947e142fea54a3024bd3e1c29eff7c (diff)
Highpriority patch
Diffstat (limited to 'dmenu.c')
-rw-r--r--dmenu.c65
1 files changed, 59 insertions, 6 deletions
diff --git a/dmenu.c b/dmenu.c
index d07e55d..9a9cc5f 100644
--- a/dmenu.c
+++ b/dmenu.c
@@ -27,14 +27,16 @@
#define TEXTW(X) (drw_fontset_getwidth(drw, (X)) + lrpad)
/* enums */
-enum { SchemeNorm, SchemeSel, SchemeOut, SchemeLast }; /* color schemes */
+enum { SchemeNorm, SchemeSel, SchemeHp, SchemeOut, SchemeLast }; /* color schemes */
struct item {
char *text;
struct item *left, *right;
- int out;
+ int out, hp;
};
+static char **hpitems = NULL;
+static int hplength = 0;
static char text[BUFSIZ] = "";
static char *embed;
static int bh, mw, mh;
@@ -63,6 +65,36 @@ static char *tempfonts;
static int (*fstrncmp)(const char *, const char *, size_t) = strncmp;
static char *(*fstrstr)(const char *, const char *) = strstr;
+static char**
+tokenize(char *source, const char *delim, int *llen) {
+ int listlength = 0;
+ char **list = malloc(1 * sizeof(char*));
+ char *token = strtok(source, delim);
+
+ while (token) {
+ if (!(list = realloc(list, sizeof(char*) * (listlength + 1))))
+ die("Unable to realloc %d bytes\n", sizeof(char*) * (listlength + 1));
+ if (!(list[listlength] = strdup(token)))
+ die("Unable to strdup %d bytes\n", strlen(token) + 1);
+ token = strtok(NULL, delim);
+ listlength++;
+ }
+
+ *llen = listlength;
+ return list;
+}
+
+static int
+arrayhas(char **list, int length, char *item) {
+ for (int i = 0; i < length; i++) {
+ int len1 = strlen(list[i]);
+ int len2 = strlen(item);
+ if (fstrncmp(list[i], item, len1 > len2 ? len2 : len1) == 0)
+ return 1;
+ }
+ return 0;
+}
+
static void
appenditem(struct item *item, struct item **list, struct item **last)
{
@@ -130,6 +162,8 @@ drawitem(struct item *item, int x, int y, int w)
{
if (item == sel)
drw_setscheme(drw, scheme[SchemeSel]);
+ else if (item->hp)
+ drw_setscheme(drw, scheme[SchemeHp]);
else if (item->out)
drw_setscheme(drw, scheme[SchemeOut]);
else
@@ -237,7 +271,7 @@ match(void)
char buf[sizeof text], *s;
int i, tokc = 0;
size_t len, textsize;
- struct item *item, *lprefix, *lsubstr, *prefixend, *substrend;
+ struct item *item, *lhpprefix, *lprefix, *lsubstr, *hpprefixend, *prefixend, *substrend;
strcpy(buf, text);
/* separate input text into tokens to be matched individually */
@@ -246,7 +280,7 @@ match(void)
die("cannot realloc %u bytes:", tokn * sizeof *tokv);
len = tokc ? strlen(tokv[0]) : 0;
- matches = lprefix = lsubstr = matchend = prefixend = substrend = NULL;
+ matches = lhpprefix = lprefix = lsubstr = matchend = hpprefixend = prefixend = substrend = NULL;
textsize = strlen(text) + 1;
for (item = items; item && item->text; item++) {
for (i = 0; i < tokc; i++)
@@ -254,14 +288,24 @@ match(void)
break;
if (i != tokc) /* not all tokens match */
continue;
/* exact matches go first, then prefixes, then substrings */
if (!tokc || !fstrncmp(text, item->text, textsize))
appenditem(item, &matches, &matchend);
+ else if (item->hp && !fstrncmp(tokv[0], item->text, len))
+ appenditem(item, &lhpprefix, &hpprefixend);
else if (!fstrncmp(tokv[0], item->text, len))
appenditem(item, &lprefix, &prefixend);
else
appenditem(item, &lsubstr, &substrend);
}
+ if (lhpprefix) {
+ if (matches) {
+ matchend->right = lhpprefix;
+ lhpprefix->left = matchend;
+ } else
+ matches = lhpprefix;
+ matchend = hpprefixend;
+ }
if (lprefix) {
if (matches) {
matchend->right = lprefix;
@@ -569,6 +613,7 @@ readstdin(void)
if (!(items[i].text = strdup(buf)))
die("cannot strdup %u bytes:", strlen(buf) + 1);
items[i].out = 0;
+ items[i].hp = arrayhas(hpitems, hplength, items[i].text);
drw_font_getexts(drw->fonts, buf, strlen(buf), &tmpmax, NULL);
if (tmpmax > inputw) {
inputw = tmpmax;
@@ -729,7 +774,9 @@ static void
usage(void)
{
fputs("usage: dmenu [-bfivP] [-l lines] [-p prompt] [-fn font] [-m monitor]\n"
- " [-nb color] [-nf color] [-sb color] [-sf color] [-w windowid]\n", stderr);
+ " [-nb color] [-nf color] [-sb color] [-sf color] [-w windowid]\n"
+ " [-hb color] [-hf color] [-hp items]\n", stderr);
+
exit(1);
}
@@ -807,8 +854,14 @@ main(int argc, char *argv[])
colortemp[2] = argv[++i];
else if (!strcmp(argv[i], "-sf")) /* selected foreground color */
colortemp[3] = argv[++i];
+ else if (!strcmp(argv[i], "-hb")) /* high priority background color */
+ colors[SchemeHp][ColBg] = argv[++i];
+ else if (!strcmp(argv[i], "-hf")) /* low priority background color */
+ colors[SchemeHp][ColFg] = argv[++i];
else if (!strcmp(argv[i], "-w")) /* embedding window id */
embed = argv[++i];
+ else if (!strcmp(argv[i], "-hp"))
+ hpitems = tokenize(argv[++i], ",", &hplength);
else
usage();
@@ -254,14 +288,24 @@ match(void)
if (i != tokc) /* not all tokens match */
continue;
- /* exact matches go first, then prefixes, then substrings */
+ /* exact matches go first, then prefixes with high priority, then prefixes, then substrings */
if (!tokc || !fstrncmp(text, item->text, textsize))
appenditem(item, &matches, &matchend);